let HtmlWebpackPlugin;
const _ = require('lodash');
const sha256 = require('sha.js')('sha256');
const path = require('path');
const fs = require('fs');

let cwd = process.cwd();

try {
  HtmlWebpackPlugin = require('html-webpack-plugin');
} catch (error) {
  throw new Error(`
    Must html-webpack-plugin >=3`)
}

class CssAssetsAttr {
  constructor(op) {
    op = op || {};
    this.op = op;
  }

  apply(compiler) {
    this.registerHooks(compiler);
  }

  registerHooks(compiler) {
    compiler.hooks.compilation.tap('AssetsAttr', (compilation) => {
      if (HtmlWebpackPlugin.getHooks) {
        HtmlWebpackPlugin.getHooks(compilation).alterAssetTags.tapAsync('asstes', (data, cb) => {
          // console.log('0000', data.assetTags);

          data.assetTags.links = this.alterAssetTagsAsync(data.assetTags.links, compilation);
          cb(null, data)
        })
      } else if (compilation.hooks.htmlWebpackPluginBeforeHtmlGeneration) {
        compilation.hooks.htmlWebpackPluginAlterAssetTags.tapAsync('asstes', (data, cb) => {
          //  console.log('0001', data.head);
          data.head = this.alterAssetTagsAsync(data.head, compilation)
          cb(null, data)
        });
      } else {
        throw new Error(`
          if assets-attr-plugin version <4
          register assets-attr-plugin after html-webpack-plugin
        `)
      }
    })
  }


  alterAssetTagsAsync(assets, compilation) {
    // console.log('1----',assets);
    // console.log(compilation);



    // attrs : 所有的css
    // bundle: 打包的css
    // online: 添加css, 同时可设置属性
    let {
      attrs = {},
        bundle = {},
        online = [],
        local = {
          root: './',
          src: [],
          chunkHash: false
        }
    } = this.op;

    let append = [],
      prepend = [];


    let configPublicPath = compilation.outputOptions.publicPath || '';
    let localRoot = local.root ? local.root : './';
    let localHref = local.href ? local.href : [];
    let localChunkHash = local.chunkHash === true ? local.chunkHash : false;
    let fileBase = path.resolve(cwd, localRoot);
    // console.log(1111,localHref);

    let localCsss = localHref.map(url => {

      let ext = path.extname(url);
      let basename = path.basename(url, ext);
      let filePath = path.resolve(fileBase, url);

      let fileContent = fs.readFileSync(filePath, {
        encoding: 'utf8'
      });

      // local输出文件名
      let fileName = '';

      if (localChunkHash) {
        let hash = sha256.update(fileContent).digest('hex').slice(0, 8);
        fileName = `${basename}-${hash}${ext}`;
      } else {
        fileName = `${basename}${ext}`;
      }


      compilation.assets[fileName] = {
        source: () => fileContent,
        size: () => fileContent.length
      }

      // 最终得到的publicPath
      url = configPublicPath + fileName;

      return {
        tagName: 'link',
        voidTag: false,
        attributes: {
          href: url,
          ...attrs,
        }
      }
    });

    online.forEach(group => {
      // console.log('6666', attrs);

      let tags = group.assets.map(elt => {
        return {
          tagName: 'link',
          voidTag: false,
          attributes: {
            href: elt,
            ...attrs,
            ...group.attrs,
          }
        }
      });

      // console.log(tags);

      if (group.append) {
        append.push(..._.flatten(tags));
      } else {
        prepend.push(..._.flatten(tags))
      }

    });

    // console.log('3333', attrs);
    // console.log('444', bundle);

    assets = assets.map(elt => {
      // console.log(elt.tagName === 'link');

      if (elt.tagName === 'link') {
        elt.attributes = {
          ...elt.attributes,
          ...attrs,
          ...bundle
        }
      }


      return elt;
    })

    // console.log(prepend, localCsss, assets, append);

    let allCss = _.concat(prepend, localCsss, assets, append);

    return allCss;

  }

}

module.exports = CssAssetsAttr
